<?php
/**
*
* @package phpBB3
* @version $Id: prime_topic_move.php,v 1.2.2 2009/05/19 12:32:00 primehalo Exp $
* @copyright (c) 2007-2008 primehalo
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Only include once.
*/
global $prime_topic_move;
if (!defined('INCLUDES_PRIME_TOPIC_MOVE_PHP'))
{
	/**
	* Constants
	*/
	define('INCLUDES_PRIME_TOPIC_MOVE_PHP', true);
	define('PRIME_TOPIC_MOVED_BIT_FIRST',	0x01);
	define('PRIME_TOPIC_MOVED_BIT_LAST',	0x02);
	define('PRIME_TOPIC_MOVED_BIT_MOVED',	0x04);
	define('PRIME_TOPIC_MOVED_BIT_SHADOW',	0x08);

	/**
	* Custom ACP option
	*/
	function prime_topic_move_acp_option($key = '', $select_id = PRIME_TOPIC_MOVED_BIT_MOVED)
	{
		global $user, $config;

		$user->add_lang('mods/prime_topic_move');
		$options_array = array(
			PRIME_TOPIC_MOVED_BIT_FIRST		=> 'PRIME_TOPIC_MOVED_DISPLAY_FIRST',
			PRIME_TOPIC_MOVED_BIT_LAST		=> 'PRIME_TOPIC_MOVED_DISPLAY_LAST',
			PRIME_TOPIC_MOVED_BIT_MOVED		=> 'PRIME_TOPIC_MOVED_DISPLAY_MOVED',
			PRIME_TOPIC_MOVED_BIT_SHADOW	=> 'PRIME_TOPIC_MOVED_DISPLAY_SHADOW'
		);
		if ($key == 'topic_moved_display')
		{
			$select = '';
			foreach ($options_array as $idx => $title)
			{
				$selected = ($config['topic_moved_display'] & $idx) ? ' checked="checked"' : '';
				$select .= '<label for="' . $key . ($idx == 1 ? '' : $idx) . '"><input type="checkbox" id="' . $key . ($idx == 1 ? '' : $idx) . '" name="topic_moved_display[' . $idx . ']"' . $selected . ' style="cursor:auto" />&nbsp;' . $user->lang[$title] . '</label><br />' . "\n";
			}
			return($select);
		}
	}

	/**
	* Class structure
	*/
	class prime_topic_move
	{
		var $users = array();
		var $forums = array();

		/**
		* Constructor
		*/
		function prime_topic_move()
		{
			$this->initialize_options();
		}

		/**
		* Set defaults for options that have not been set.
		*/
		function initialize_options()
		{
			global $config;

			if (!isset($config['topic_moved_display']))
			{
				set_config('topic_moved_display', PRIME_TOPIC_MOVED_BIT_MOVED);
			}
		}

		/**
		* Display the ACP options.
		*/
		function display_acp_options(&$display_vars, $mode)
		{
			global $config, $user;

			if ($mode == 'features')
			{
				$user->add_lang('mods/prime_topic_move');
				$display_copy = $display_vars['vars'];
				$display_vars['vars'] = array();
				foreach ($display_copy as $key => $val)
				{
					$display_vars['vars'][$key] = $val;
					if ($key == 'allow_birthdays')	// Insert our option after this one
					{
						$display_vars['vars']['topic_moved_display'] = array('lang' => 'PRIME_TOPIC_MOVED_DISPLAY', 'validate' => 'string', 'type' => 'custom', 'function' => 'prime_topic_move_acp_option', 'params' => array('{KEY}', '{CONFIG_VALUE}'), 'explain' => true);
					}
				}
			}
		}

		/**
		* Merge the checkbox array data into one variable.
		*/
		function merge_options($config_name = '', &$config_array)
		{
			if ($config_name == 'topic_moved_display')
			{
				$merged = 0;
				$checkboxes = request_var('topic_moved_display', array('' => ''));
				foreach ($checkboxes as $k => $v)
				{
					$merged |= $k;
				}
				unset($checkboxes);
				$config_array[$config_name] = $merged;
			}
		}

		/**
		*/
		function mcp_sql_setup($forum_id)
		{
			global $user;

			$sql_array = array(
				'topic_moved_from'		=> $forum_id,
				'topic_moved_user'		=> $user->data['user_id'],
				'topic_moved_time'		=> time(),
				'topic_moved_reason'	=> utf8_normalize_nfc(request_var('moved_reason', '', true)),
			);
			return $sql_array;
		}

		/**
		*/
		function mcp_sql_execute(&$topic_info, &$sql_array)
		{
			global $db;

			$sql_array['topic_moved_last_post_id'] = $topic_info['topic_last_post_id'];
			$sql = 'UPDATE ' . TOPICS_TABLE
					. ' SET ' . $db->sql_build_array('UPDATE', $sql_array)
					. ' WHERE topic_id=' . $topic_info['topic_id']
			;
			$db->sql_query($sql);
		}

		/**
		* Assign our template variables for the confirmation page.
		*/
		function mcp_assign_template_vars($topic_ids, $forum_id = 0)
		{
			global $user, $template;

			$user->add_lang('mods/prime_topic_move');
			$template->assign_var('L_PRIME_TOPIC_MOVE_ENTER_REASON', (sizeof($topic_ids) == 1) ? $user->lang['PRIME_TOPIC_MOVE_ENTER_REASON'] : $user->lang['PRIME_TOPIC_MOVE_ENTER_REASONS']);
		}

		/**
		*/
		function viewforum_insert_message(&$row, $forum_id)
		{
			global $config, $template;

			if (isset($row['topic_moved_from']) && ($row['topic_moved_from'] == $forum_id) && ($config['topic_moved_display'] & PRIME_TOPIC_MOVED_BIT_SHADOW) && ($msg = $this->get_message($row)))
			{
				$template->alter_block_array('topicrow', array('TOPIC_MOVED_MESSAGE' => $msg), true, 'change');
			}
		}

		/**
		*/
		function viewtopic_insert_message(&$postrow, &$topic_data, $i, $end, $viewtopic_url = '')
		{
			global $config;

			if (!empty($topic_data['topic_moved_user']) && !empty($topic_data['topic_moved_from']) && !empty($topic_data['topic_moved_time']))
			{
				$on_first_post = (($config['topic_moved_display'] & PRIME_TOPIC_MOVED_BIT_FIRST) && ($postrow['POST_ID'] == $topic_data['topic_first_post_id']));	//($i == 0)
				$on_final_post = (($config['topic_moved_display'] & PRIME_TOPIC_MOVED_BIT_LAST ) && ($postrow['POST_ID'] == $topic_data['topic_last_post_id']));	//($i == $end - 1)
				$on_moved_post = (($config['topic_moved_display'] & PRIME_TOPIC_MOVED_BIT_MOVED) && ($postrow['POST_ID'] == $topic_data['topic_moved_last_post_id']));
				if ($on_first_post || $on_final_post || $on_moved_post)
				{
					$postrow['TOPIC_MOVED_MESSAGE'] = $this->get_message($topic_data, $viewtopic_url);
				}
			}
		}


		function get_message(&$topic_data, $viewtopic_url = null)
		{
			if (empty($topic_data['topic_moved_user']) && empty($topic_data['topic_moved_from']) && empty($topic_data['topic_moved_time']))
			{
				return '';
			}
			global $config, $user, $auth, $template, $db, $phpEx, $phpbb_root_path;

			$mode		= ($viewtopic_url === null) ? 'forum' : 'topic';
			$moved_from	= isset($topic_data['topic_moved_from']) ? $topic_data['topic_moved_from'] : 0;
			$user_id	= isset($topic_data['topic_moved_user']) ? $topic_data['topic_moved_user'] : 0;
			$forum_id	= ($mode == 'forum') ? $topic_data['forum_id'] : $moved_from;
			$to_or_from	= ($mode == 'forum') ? 'TOPIC_MOVED_TO' : 'TOPIC_MOVED_FROM';
			$can_undo	= ($mode == 'topic' && $auth->acl_get('f_list', $moved_from) && $auth->acl_get('m_move', $topic_data['forum_id']) && $topic_data['topic_status'] != ITEM_MOVED) ? true : false;

			// We need information about the user who moved the post and about the forum that it was moved from/to.
			if (empty($this->users[$user_id]) || empty($this->forums[$forum_id]))
			{
				$user->add_lang('mods/prime_topic_move');
				$sql = 'SELECT DISTINCT u.username, u.user_colour, f.forum_name'
						. ' FROM ' . USERS_TABLE . ' u, ' . FORUMS_TABLE . ' f'
						. ' WHERE user_id=' . $user_id . ' AND forum_id=' . $forum_id
				;
				$result = $db->sql_query($sql);
				$moved_info = $db->sql_fetchrow($result);
				$db->sql_freeresult($result);
				$this->users[$user_id] = array('username' => $moved_info['username'], 'user_colour' => $moved_info['user_colour']);
				$this->forums[$forum_id] = array('forum_name' => $moved_info['forum_name']);
			}
			else
			{
				$moved_info = $this->users[$user_id] + $this->forums[$forum_id];
			}

			// Setup the variables.
			$msg['moved']	= $user->lang['PRIME_TOPIC_MOVED_MESSAGE'];
			$msg['forum']	= (empty($moved_info['forum_name']) || !$auth->acl_get('f_list', $forum_id)) ? '' : '<a href="' . append_sid("{$phpbb_root_path}viewforum.$phpEx", "f={$forum_id}", true) . '">' . $moved_info['forum_name'] . '</a>';
			$msg['forum']	= empty($msg['forum']) ? '' : sprintf($user->lang['PRIME_' . $to_or_from], $msg['forum']);
			$msg['by']		= empty($moved_info['username']) ? $user->lang['PRIME_TOPIC_MOVED_NO_USER'] : get_username_string('full', $user_id, $moved_info['username'], $moved_info['user_colour']);
			$msg['by']		= empty($msg['by']) ? '' : sprintf($user->lang['PRIME_TOPIC_MOVED_BY'], $msg['by']);
			$msg['on']		= empty($topic_data['topic_moved_time']) ? '' : $user->format_date($topic_data['topic_moved_time']);
			$msg['on']		= empty($msg['on']) ? '' : sprintf($user->lang['PRIME_TOPIC_MOVED_ON'], $msg['on']);
			$msg['undo']	= $can_undo ? ' <a href="' . append_sid("{$phpbb_root_path}mcp.$phpEx", "f={$topic_data['forum_id']}&amp;t={$topic_data['topic_id']}&amp;quickmod=1&amp;action=move&amp;to_forum_id={$forum_id}&amp;redirect=" . urlencode(str_replace('&amp;', '&', $viewtopic_url)), true, $user->session_id) . '" class="post_unmove">' . $user->lang['PRIME_TOPIC_MOVED_UNDO'] . '</a>' : '';
			$template->assign_vars(array(
				'TOPIC_MOVED_MODE'		=> $mode,
				'TOPIC_MOVED_MESSAGE'	=> $msg['moved'],
				$to_or_from				=> $msg['forum'],
				'TOPIC_MOVED_BY' 		=> $msg['by'],
				'TOPIC_MOVED_ON'		=> $msg['on'],
				'TOPIC_MOVED_REASON'	=> isset($topic_data['topic_moved_reason']) ? $topic_data['topic_moved_reason'] : '',
				'TOPIC_MOVED_UNDO'		=> $msg['undo'],
			));
			$template->set_filenames(array('moved_msg' => 'prime_topic_move.html'));
			$moved_msg = $template->assign_display('moved_msg');
			return($moved_msg);
		}
	}
	// End class

	$prime_topic_move = new prime_topic_move();
}
?>